
from modeltranslation.translator import translator

from orders.models import PaymentMethod, DeliveryMethod, OrderStatus


translator.register(PaymentMethod, fields=['name'])
translator.register(DeliveryMethod, fields=['name'])
translator.register(OrderStatus, fields=['name'])




from django.apps import apps
from django.contrib import admin
from django.utils.timezone import localtime
from django.utils.translation import gettext_lazy as _

from cap.decorators import template_list_item
from modeltranslation.admin import TranslationAdmin

from orders.constants import DAYS_OF_WEEK
from orders.models import Order, OrderStatus, PaymentMethod, DeliveryMethod


@admin.register(Order)
class OrderAdmin(admin.ModelAdmin):

    list_display = ['order_tag', 'details_tag', 'products_tag']

    list_filter = ['status']

    search_fields = [
        'first_name', 'last_name', 'middle_name', 'address', 'mobile',
        'comment'
    ]

    def has_add_permission(self, request):
        return False

    @template_list_item('orders/admin/details.html', _('Details'))
    def details_tag(self, obj):
        return {
            'object': obj
        }

    @template_list_item('orders/admin/products.html', _('Products'))
    def products_tag(self, obj):
        return {
            'object': obj,
            'is_clothes_app_enabled': apps.is_installed('clothes')
        }

    @template_list_item('orders/admin/order.html', _('Order'))
    def order_tag(self, obj):

        created = localtime(obj.created)

        return {
            'object': obj,
            'day': DAYS_OF_WEEK[created.weekday()],
            'created': created.strftime('%d.%m.%Y %H:%M')
        }


admin.site.register(OrderStatus, TranslationAdmin)
admin.site.register(PaymentMethod, TranslationAdmin)
admin.site.register(DeliveryMethod, TranslationAdmin)



# Generated by Django 4.1.10 on 2023-08-28 07:56

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import orders.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DeliveryMethod',
            fields=[
                ('id', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('name_uk', models.CharField(max_length=255, null=True, verbose_name='Name')),
                ('name_ru', models.CharField(max_length=255, null=True, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Delivery method',
                'verbose_name_plural': 'Delivery methods',
            },
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=255, verbose_name='First name')),
                ('last_name', models.CharField(max_length=255, verbose_name='Last name')),
                ('middle_name', models.CharField(blank=True, max_length=255, verbose_name='Middle name')),
                ('address', models.CharField(blank=True, max_length=255, verbose_name='Address')),
                ('mobile', models.CharField(max_length=255, verbose_name='Mobile number')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('comment', models.TextField(blank=True, max_length=1000, verbose_name='Comment')),
                ('decline_reason', models.TextField(blank=True, max_length=1000, verbose_name='Decline reason')),
                ('hash', models.CharField(default=orders.models._generate_hash, max_length=10, unique=True)),
                ('delivery_method', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='orders.deliverymethod', verbose_name='Delivery method')),
            ],
            options={
                'verbose_name': 'Order',
                'verbose_name_plural': 'Orders',
            },
        ),
        migrations.CreateModel(
            name='OrderStatus',
            fields=[
                ('id', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('name_uk', models.CharField(max_length=255, null=True, verbose_name='Name')),
                ('name_ru', models.CharField(max_length=255, null=True, verbose_name='Name')),
                ('is_default', models.BooleanField(default=False, verbose_name='Default')),
            ],
            options={
                'verbose_name': 'Status',
                'verbose_name_plural': 'Statuses',
            },
        ),
        migrations.CreateModel(
            name='PaymentMethod',
            fields=[
                ('id', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('name_uk', models.CharField(max_length=255, null=True, verbose_name='Name')),
                ('name_ru', models.CharField(max_length=255, null=True, verbose_name='Name')),
            ],
            options={
                'verbose_name': 'Payment method',
                'verbose_name_plural': 'Payment methods',
            },
        ),
        migrations.CreateModel(
            name='OrderedProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('product_id', models.IntegerField(blank=True, null=True, verbose_name='Product ID')),
                ('product_name', models.CharField(max_length=255, verbose_name='Product name')),
                ('product_code', models.CharField(max_length=255, verbose_name='Product code')),
                ('product_price', models.FloatField(verbose_name='Product price')),
                ('product_printable_price', models.CharField(max_length=100, verbose_name='Product printable price')),
                ('product_currency', models.PositiveIntegerField(verbose_name='Product currency')),
                ('product_logo', models.ImageField(blank=True, null=True, upload_to='ordered_product_logos', verbose_name='Product logo')),
                ('qty', models.PositiveIntegerField(default=1, verbose_name='Quantity')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='orders.order', verbose_name='Order')),
            ],
            options={
                'verbose_name': 'Ordered product',
                'verbose_name_plural': 'Ordered products',
            },
        ),
        migrations.AddField(
            model_name='order',
            name='payment_method',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='orders.paymentmethod', verbose_name='Payment method'),
        ),
        migrations.AddField(
            model_name='order',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='orders.orderstatus', verbose_name='Status'),
        ),
        migrations.AddField(
            model_name='order',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='orders', to=settings.AUTH_USER_MODEL, verbose_name='Owner'),
        ),
    ]




from django import forms
from django.utils.translation import gettext_lazy as _

from orders.models import Order


class CheckoutForm(forms.ModelForm):

    city = forms.CharField(
        label=_('City'),
        required=False
    )

    warehouse = forms.CharField(
        label=_('Warehouse'),
        required=False
    )

    def clean_mobile(self):
        mobile = self.cleaned_data['mobile']
        if mobile.startswith('+3800'):
            raise forms.ValidationError(
                _('Phone number could not start with `+3800`'))
        return mobile

    class Meta:
        model = Order
        fields = [
            'first_name', 'last_name', 'middle_name', 'payment_method',
            'delivery_method', 'mobile', 'comment'
        ]




from django.utils.translation import gettext_lazy as _


DAYS_OF_WEEK = [
    _('Monday'),
    _('Tuesday'),
    _('Wednesday'),
    _('Thursday'),
    _('Friday'),
    _('Saturday'),
    _('Sunday')
]

ORDER_STATUS_NEW = 'new'

PAYMENT_METHOD_PRIVAT24 = 'privat24'




from django.apps import apps
from django.conf import settings
from django.contrib.humanize.templatetags.humanize import intcomma
from django.db import models
from django.utils.crypto import get_random_string
from django.utils.translation import gettext_lazy as _

from orders.constants import PAYMENT_METHOD_PRIVAT24


def _generate_hash():
    return get_random_string(length=10)


class OrderStatus(models.Model):

    id = models.CharField(primary_key=True, max_length=100)

    name = models.CharField(_('Name'), max_length=255)

    is_default = models.BooleanField(_("Default"), default=False)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Status')
        verbose_name_plural = _('Statuses')


class PaymentMethod(models.Model):

    id = models.CharField(primary_key=True, max_length=100)

    name = models.CharField(_('Name'), max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Payment method')
        verbose_name_plural = _('Payment methods')


class DeliveryMethod(models.Model):

    id = models.CharField(primary_key=True, max_length=100)

    name = models.CharField(_('Name'), max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Delivery method')
        verbose_name_plural = _('Delivery methods')


class Order(models.Model):

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        models.SET_NULL,
        related_name='orders',
        verbose_name=_('Owner'),
        null=True,
        blank=True
    )

    status = models.ForeignKey(
        OrderStatus,
        models.CASCADE,
        verbose_name=_('Status')
    )

    payment_method = models.ForeignKey(
        PaymentMethod,
        models.CASCADE,
        verbose_name=_('Payment method'),
    )

    delivery_method = models.ForeignKey(
        DeliveryMethod,
        models.CASCADE,
        verbose_name=_('Delivery method'),
    )

    first_name = models.CharField(_('First name'), max_length=255)

    last_name = models.CharField(_('Last name'), max_length=255)

    middle_name = models.CharField(
        _('Middle name'), max_length=255, blank=True)

    address = models.CharField(_('Address'), max_length=255, blank=True)

    mobile = models.CharField(_('Mobile number'), max_length=255)

    created = models.DateTimeField(
        _('Date created'), auto_now_add=True, editable=False)

    comment = models.TextField(_('Comment'), max_length=1000, blank=True)

    decline_reason = models.TextField(
        _('Decline reason'), max_length=1000, blank=True)

    hash = models.CharField(
        max_length=10,
        default=_generate_hash,
        unique=True)

    def __str__(self):
        return self.printable_name

    @property
    def printable_name(self):
        return '{} #{}'.format(_('Order'), self.id)

    @property
    def full_name(self):
        return '{} {} {}'.format(
            self.last_name, self.first_name, self.middle_name)

    @property
    def total(self):
        return sum([i.subtotal for i in self.items.all()])

    @property
    def printable_total(self):
        return intcomma(self.total)

    def is_liqpay_payment(self):
        return self.is_paynow_form_visible() and apps.is_installed('liqpay')

    def is_paynow_form_visible(self):
        return self.payment_method.code == PAYMENT_METHOD_PRIVAT24

    class Meta:
        verbose_name = _('Order')
        verbose_name_plural = _('Orders')


class OrderedProduct(models.Model):

    order = models.ForeignKey(
        Order,
        verbose_name=_('Order'),
        related_name='items',
        on_delete=models.CASCADE)

    product_id = models.IntegerField(_('Product ID'), blank=True, null=True)
    product_name = models.CharField(_("Product name"), max_length=255)
    product_code = models.CharField(_("Product code"), max_length=255)
    product_price = models.FloatField(_("Product price"))
    product_printable_price = models.CharField(
        _("Product printable price"), max_length=100)
    product_currency = models.PositiveIntegerField(_("Product currency"))
    product_logo = models.ImageField(
        _("Product logo"),
        blank=True,
        null=True,
        upload_to='ordered_product_logos'
    )

    qty = models.PositiveIntegerField(_('Quantity'), default=1)

    def __str__(self):
        return self.product_name

    @property
    def subtotal(self):
        return self.product_price * self.qty

    def printable_subtotal(self):
        return intcomma(self.subtotal)

    class Meta:
        verbose_name = _('Ordered product')
        verbose_name_plural = _('Ordered products')




from django.apps import apps, AppConfig
from django.utils.translation import gettext_lazy as _


class OrdersConfig(AppConfig):
    name = 'orders'
    verbose_name = _("Orders")

    def ready(self):
        if not apps.is_installed('djmail'):
            raise Exception('`mp-orders` depends on `mp-email`')

        if not apps.is_installed('cart'):
            raise Exception('`mp-orders` depends on `mp-cart`')
